@@ -6,7 +6,7 @@
 #' the data in the particular case of a single treated unit.
 #'
 #' The names of the output matrices follow the terminology proposed in
-#' \href{https://nppackages.github.io/references/Cattaneo-Feng-Titiunik_2021_JASA.pdf}{Cattaneo, Feng, Palomba and Titiunik (2022)}  (UPDATE LINK).
+#' \href{https://nppackages.github.io/references/Cattaneo-Feng-Titiunik_2021_JASA.pdf}{Cattaneo, Feng, Palomba and Titiunik (2022)}.
 #'
 #' Companion \href{https://www.stata.com/}{Stata} and \href{https://www.python.org/}{Python} packages are
 #' described in \href{https://arxiv.org/abs/2202.05984}{Cattaneo, Feng, Palomba, and Titiunik (2022)}.
@@ -664,6 +664,7 @@ scdataMulti <- function(df,
                       specs = specs)
 
   } else {
+    if (effect == "time") P.stacked <- as.matrix(P.stacked) # in this case P.stacked is a data.frame
     df.sc <-     list(A = A.stacked,
                       B = B.stacked,
                       C = C.stacked,
